/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package db

import (
	"database/sql"
	"fmt"
	_ "github.com/lib/pq"
)

type UserInfo struct {
	Name   string
	Email  string
	Is_aod bool
}

/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

/**
 * Establish a connection to the database
 *
 * @param db_info A DbInfo type with information to connect to the database
 * @return A DB object with an open connection to the database
 **/
func getConnection(db_info DbInfo) *sql.DB {
	// Get db info and build string to get connection
	db_info = getDbLoginFromProperties(db_info)

	ssl_mode := "disable"
	if db_info.Is_ssl {
		ssl_mode = "require"
	}
	conn_info := fmt.Sprintf("host=%s port=%d user=%s password=%s dbname=%s sslmode=%s", db_info.Host, db_info.Port, db_info.User, db_info.Password, db_info.Dbname, ssl_mode)

	db, err := sql.Open("postgres", conn_info)
	checkError(err)

	err = db.Ping()
	checkError(err)

	return db
}

/**
 * Add a user to the qa_staff table in the database
 *
 * @param name a string with the name of the person to be added
 * @param is_aod a bool to check if the user is an AOD or not
 * @param db_info a DbInfo type with information to connect to the database
 **/
func AddUser(user_info UserInfo, db_info DbInfo) {
	// Get a connection to the database
	db := getConnection(db_info)

	defer db.Close()

	group := "DA"
	if user_info.Is_aod {
		group = "AOD"
	}

	// Add the user
	insertStatement := `insert into "qa_staff"("user_name", "group", "available", "email") values($1, $2, $3, $4)`
	_, err := db.Exec(insertStatement, user_info.Name, group, true, user_info.Email)
	checkError(err)

	fmt.Println("Added " + user_info.Name + " to qa_staff")
}

/**
 * Remove a user from the qa_staff table in the database
 *
 * @param name a string with the name of the person to be removed
 * @param db_info a DbInfo type with information to connect to the database
 **/
func RemoveUser(user_info UserInfo, db_info DbInfo) {
	// Get a connection to the database
	db := getConnection(db_info)

	defer db.Close()

	// Remove the user
	deleteStatement := `delete from "qa_staff" where "user_name"=$1`
	_, err := db.Exec(deleteStatement, user_info.Name)
	checkError(err)

	fmt.Println("Removed " + user_info.Name + " from qa_staff")
}
