#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Solicitor: uses metadata.json to determine input parameters for ingestion
"""
import http
import json
import logging
import pathlib
from typing import List, Union

import requests

from ingest_envoy.utilities import IngestType, VLASSIngestType


INVALID_INITIAL_VERSION = "Initial version not valid for ingest"


class Solicitor:
    def __init__(self, arg_type: Union[IngestType, VLASSIngestType], urls: List, filename: str = None):
        if filename is not None:
            self.filename = filename
            self.metadata = self.solicit_contents(self.filename)

        self.logger = logging.getLogger("ingest_envoy")
        self.argument = arg_type
        self.capability_url = urls[0]
        self.workflow_url = urls[1]

    @staticmethod
    def solicit_contents(filename: str) -> dict:
        """
        read provided metadata file

        :return: dict of parameters
        """
        with open(filename) as file:
            metadata = json.loads(file.read())
            return metadata

    def solicit_aux_metadata(self) -> dict:
        """
        read aux_image_metadata file

        :return: dict of parameters
        """
        return self.solicit_contents(filename="aux_image_metadata.json")

    def solicit_workflow_directory_name(self) -> str:
        """
        determine the original workflow processing directory's name without the path

        :return: directory name string
        """
        destination_path = pathlib.Path(self.metadata["destinationDirectory"])
        return destination_path.stem

    def solicit_initial_directory_name(self, wf_id: int) -> str:
        """
        Determine the initial directory if more than one version was needed

        :param wf_id: the workflow id of the workflow being ingested
        :return: directory name string
        """
        current_execution_url = f"{self.capability_url}/capability/request/execution/{wf_id}"
        current_execution = requests.get(current_execution_url)

        initial_execution_url = (
            f"{self.capability_url}/capability/request/"
            f"{current_execution.json()['capability_request_id']}/version/1/execution"
        )
        initial_execution = requests.get(initial_execution_url)

        wf_request_url = (
            f"{self.workflow_url}/workflows/std_calibration/requests/"
            f"{initial_execution.json()['current_workflow_request_id']}"
        )
        wf_request = requests.get(wf_request_url)

        if self.verify_valid_initial_version(wf_request.json()):
            initial_path = pathlib.Path(wf_request.json()["results_dir"])
            return initial_path.stem
        else:
            self.logger.error("Error: Invalid initial version! No weblog found. Running Single Version Ingestion...")
            return INVALID_INITIAL_VERSION

    def verify_valid_initial_version(self, wf_request: dict) -> bool:
        """
        Ping the weblog display endpoint to verify the existence of an initial version weblog
        * Ingestion fails if this is unavailable
        :param wf_request: the initial workflow request to verify
        :return: True if weblog exists, False otherwise
        """

        srdp_flag = wf_request["argument"]["metadata"]["is_srdp"]

        # only run ingestion with initial version weblog if this is SRDP
        if srdp_flag is True:
            # if this is SRDP, verify the initial weblog exists
            get_weblog_url = (
                f"{self.workflow_url}/workflows/std_calibration/requests/"
                f"{wf_request['current_workflow_request_id']}/weblog"
            )
            response = requests.get(get_weblog_url)

            if response.status_code == http.HTTPStatus.FOUND:
                return True

        return False

    def solicit_processing_creation_time(self) -> str:
        """
        transform a datetime into and NGAS friendly representation
        Example: incoming datetime: 2021-07-06T21:50:48
                 transformation for NGAS friendliness: 2021_07_06T21_50_48

        :return: string
        """

        creation_time = self.metadata["creationTime"].replace("-", "_").replace(":", "_")
        return creation_time

    def get_general_params(self) -> dict:
        """
        Determine general ingestion parameters (WS only)
        :return: dict
        """

        gen = {
            "requestId": self.metadata["systemId"],
            "telescope": self.metadata["projectMetadata"]["telescope"],  # all, needed by manifest generator
            "project": self.metadata["projectMetadata"]["projectCode"],  # needed for post ingestion messaging
            "workflowName": self.metadata["workflowName"],  # all, needed by ingestion type check
            "processingStart": self.solicit_processing_creation_time(),  # cal/image
            "workflowDir": self.solicit_workflow_directory_name(),  # cal/image
            "initialVersionDir": self.solicit_initial_directory_name(self.metadata["systemId"]),  # currently, only cal
        }

        if gen["workflowDir"] == gen["initialVersionDir"] or gen["initialVersionDir"] == INVALID_INITIAL_VERSION:
            gen["multiVersion"] = False
        else:
            gen["multiVersion"] = True

        return gen

    def solicit_calibration_params(self) -> dict:
        """
        Determine calibration specific parameters (WS Only)

        :return: dict
        """

        general = self.get_general_params()

        cal = {
            "sdmId": self.metadata["fileSetIds"],  # calibration only
            "spl": self.metadata["productLocator"],  # calibration only
        }

        return {**general, **cal}

    def solicit_image_params(self) -> dict:
        """
        Determine image specific parameters (WS Only)

        :return: dict
        """
        general = self.get_general_params()

        img = {"calSpl": self.metadata["calProductLocator"]}  # image only

        return {**general, **img}

    def solicit_seci_params(self) -> dict:
        """
        Determine SECI image specific parameters

        :return: dict
        """
        aux = self.solicit_aux_metadata()

        aux2 = {"project": aux["project_code"]}

        seci = {
            "telescope": "EVLA",
            "workflowName": "vlass_seci",
            "multiVersion": False,
        }

        return {**aux2, **seci}

    def solicit_parameters(self) -> dict:
        """
        Create a dict of parameters needed for ingestion

        :return: parameters dict
        """

        if self.argument == IngestType.CAL:
            return self.solicit_calibration_params()
        elif self.argument == IngestType.IMG:
            return self.solicit_image_params()
        elif self.argument == VLASSIngestType.SECI:
            return self.solicit_seci_params()
